import ssl, socket, html, collections, urllib.parse, time, math
import ejcli._http.ejudge.ej371, ejcli._http.ejudge.ej373, ejcli._http.html2md as html2md, ejcli._http.types as bjtypes
from ejcli._http.base import Backend
from ejcli.error import EJError

def _http_header_capitalize(h):
    return '-'.join(i[:1].upper()+i[1:].lower() for i in h.split('-'))

def do_http(url, method, headers={}, data=b''):
    if '://' not in url:
        raise EJError("Invalid URL")
    proto, path = url.split('://', 1)
    if proto not in ('http', 'https'):
        raise EJError("Not an HTTP url: " + url)
    if '/' not in path: path += '/'
    s_host, path = path.split('/', 1)
    path = '/' + path
    host = s_host
    if ':' in host:
        host, port = host.rsplit(':', 1)
        port = int(port)
    else: port = 80 if proto == 'http' else 443
    if host.startswith('[') and host.endswith(']'): host = host[1:-1]
    sock = socket.create_connection((host, port))
    if proto == 'https':
        sock = ssl.create_default_context().wrap_socket(sock, server_hostname=host)
    headers['Host'] = s_host
    if data:
        headers['Content-Length'] = len(data)
    request = ['%s %s HTTP/1.1' % (method, path)]
    for k, v in headers.items():
        request.append(str(k) + ': ' + str(v))
    request.append('')
    request.append('')
    sock.sendall('\r\n'.join(request).encode('utf-8'))
    if data:
        sock.sendall(data)
    def readline():
        ans = b''
        while not ans.endswith(b'\n'): ans += sock.recv(1)
        return ans
    v, c, *exp = readline().decode('utf-8').split()
    resp_headers = []
    while True:
        l = readline().decode('utf-8').strip()
        if l == '': break
        k, v = l.split(': ', 1)
        resp_headers.append((_http_header_capitalize(k), v))
    rhd = dict(resp_headers)
    if 'Content-Length' in rhd:
        data = b''
        while len(data) < int(rhd['Content-Length']):
            data += sock.recv(int(rhd['Content-Length']) - len(data))
    elif rhd.get('Transfer-Encoding', None) == 'chunked':
        data = b''
        while True:
            l = int(readline().decode('ascii'), 16)
            data2 = b''
            while len(data2) < l:
                data2 += sock.recv(l - len(data2))
            data += data2
            readline()
            if l == 0: break
    else:
        data = b''
        nfails = 0
        while nfails < 100:
            try: chunk = sock.recv(1 << 20)
            except socket.error: break
            data += chunk
            if len(chunk) == 0: break
    sock.close()
    return (int(c), rhd, data)

#try: import requests
#except ImportError: pass
#else:
#   def do_http(url, method, headers, data=b''):
#       if method == 'GET': data = None
#       it = requests.request(method, url, data=data, headers=headers, allow_redirects=False)
#       return (it.status_code, it.headers, it.content)

def get(url, headers={}):
    return do_http(url, 'GET', headers)

def post(url, data, headers={}):
    if isinstance(data, dict):
        l = []
        for k, v in data.items():
            k += '='
            for c in str(v):
                if c.lower() in 'abcdefghijklmnopqrstuvwxyz0123456789':
                    k += c
                else:
                    k += ''.join(map('%%%02x'.__mod__, c.encode('utf-8')))
            l.append(k)
        data = '&'.join(l)
    if isinstance(data, str):
        data = data.encode('utf-8')
    return do_http(url, 'POST', headers, data)

def contest_name(url):
    code, headers, data = get(url)
    if code != 200:
        raise EJError("Page retrieval failed.")
    try: return html.unescape(data.decode('utf-8', 'replace').split('<title>', 1)[1].split(' [', 1)[1].split(']</title>', 1)[-2])
    except IndexError: return None

class Ejudge(Backend):
    @staticmethod
    def detect(url):
        return url.startswith('http://') or url.startswith('https://')
    def __init__(self, url, login, password):
        Backend.__init__(self)
        url0 = url
        url = url.replace('/new-register?', '/new-client?')
        url = url.replace('/ej/team?', '/ej/user?')
        contest_id = url.split("contest_id=")[1].split("&")[0]
        self.contest_id = int(contest_id)
        base_url = url.split("?")[0]
        code, headers, data = post(url0.split("?")[0], {'contest_id': contest_id, 'locale_id': 0, 'login': login, 'password': password, 'action_213': ''})
        if code != 302:
            raise EJError("Login failed.")
        rhd = dict(headers)
        base_url = rhd['Location'].split('&')[0]
        base_url = base_url.replace('/new-register?', '/new-client?')
        if 'new-client?SID=' in base_url:
            urls = ej371.get_urls(base_url)
        elif any(i in base_url for i in ('/user/', '/client/', '/register/', '/register?SID=')):
            urls = ej373.get_urls(base_url)
        else:
            raise EJError("Unknown ejudge version.")
        self.urls = urls
        self.cookie = rhd["Set-Cookie"].split(";")[0]
        self._get_cache = {}
    def _cache_get(self, url, cookie=True):
        with self.cache_lock:
            if url in self._get_cache:
                return self._get_cache[url]
        ans = get(url, {'Cookie': self.cookie} if cookie else {})
        with self.cache_lock:
            if self.caching: self._get_cache[url] = ans
        return ans
#   @staticmethod
#   def _task_list(data):
#       column_count = data.count('<th ')
#       if column_count == 0: return []
#       splitted = data.split('<td class="b1">')[1:]
#       if not splitted: splitted = [i.split('>', 1)[-1].split('</div>', 1)[0] for i in data.split('<div class="prob', 1)]
#       print(data)
#       data = []
#       for x in splitted[::column_count]:
#           x = x.split("</td>")[0]
#           if x.startswith('<a href="') and x.endswith('</a>'):
#               x = x.split('"', 2)[2].split('>', 1)[1][:-4]
#           data.append(html.unescape(x))
#       return data
    @staticmethod
    def _task_ids(data):
        try:
            data = data.split('<tr id="probNavTopList"><td width="100%" class="nTopNavList"><ul class="nTopNavList"><li class="first-rad">', 1)[1].split('\n', 1)[0]
        except IndexError:
            try:
                data = data.split('<td class="b0" id="probNavRightList" valign="top">', 1)[1].split('\n', 1)[0]
            except IndexError: return []
        splitted = data.split('<a class="tab" href="')[1:]
        data = [x.split('"', 1) for x in splitted]
        ans = []
        for i in data:
            ans.append((int(i[0].split('prob_id=', 1)[1]), html.unescape(i[1].split('>', 1)[-1].split('</a>', 1)[0])))
        return ans
    def tasks(self):
        code, headers, data = self._cache_get(self.urls['summary'])
        if b'<input type="submit" name="action_35" value="Change!" />' in data:
            raise EJError("Password change is required.")
        if code != 200:
            raise EJError("Failed to fetch task list.")
        data = data.decode('utf-8')
        #tl = self._task_list(data)
        ti = self._task_ids(data)
        #if len(tl) < len(ti): tl.extend([None]*(len(ti)-len(tl)))
        #else: ti.extend([None]*(len(tl)-len(ti)))
        return [bjtypes.task_t(i, j, None) for i, j in ti]
    def submissions(self):
        code, headers, data = self._cache_get(self.urls['submissions'])
        if b'<input type="submit" name="action_35" value="Change!" />' in data:
            raise EJError("Password change is required.")
        if code != 200:
            raise EJError("Failed to fetch submission list.")
        ths = [i.split('</th>', 1)[0] for i in data.decode('utf-8').split('<th class="b1">')[1:]]
        w = len(ths)
        if w == 0: return []
        splitted = data.decode('utf-8').split('<td class="b1">')[1:]
        data = [x.split("</td>")[0] for x in splitted]
        run_ids = list(map(lambda x:(int(x[:-1]) if x[-1:] == '#' else int(x)), data[ths.index('Run ID')::w]))
        task_ids = data[ths.index('Problem')::w]
        if 'Result' in ths:
            statuses = data[ths.index('Result')::w]
        else:
            statuses = [None]*len(run_ids)
        if 'Score' in ths:
            scores = []
            for i in data[ths.index('Score')::w]:
                i = i.strip()
                if i.startswith('<b>'): i = i[3:].split('</b>', 1)[0] # TODO: report score_ex in submission_stats
                i = i.strip()
                if i in ('', 'N/A', '&nbsp;'): scores.append(None)
                else: scores.append(int(i))
        else:
            scores = [None]*len(run_ids)
        if 'Tests passed' in ths:
            oktests = []
            for i in data[ths.index('Tests passed')::w]:
                i = i.strip()
                if i.startswith('<b>'): i = i[3:]
                if i.endswith('</b>'): i = i[:-4]
                i = i.strip()
                if i in ('', 'N/A', '&nbsp;'): oktests.append(None)
                else: oktests.append(int(i))
        else:
            oktests = [None]*len(run_ids)
        assert len(run_ids) == len(task_ids) == len(statuses) == len(scores) == len(oktests)
        return [bjtypes.submission_t(i, j, k, l, m) for i, j, k, l, m in zip(run_ids, task_ids, statuses, scores, oktests)]
    def submission_protocol(self, id):
        code, headers, data = self._cache_get(self.urls['protocol'].format(run_id=id))
        if b'<input type="submit" name="action_35" value="Change!" />' in data:
            raise EJError("Password change is required.")
        if code != 200:
            raise EJError("Failed to fetch testing protocol.")
        w = data.count(b'<th ')
        if w == 0: return []
        splitted = data.decode('utf-8').split('<td class="b1">')[1:]
        data = [x.split("</td>")[0] for x in splitted]
        statuses = [i[:-7].split('>')[-1] for i in data[1::w]]
        tls = []
        for i in map(html.unescape, data[2::w]):
            if i.startswith('>'): i = i[1:]
            tls.append(float(i))
        assert len(statuses) == len(tls)
        return [bjtypes.test_t(i, {'time_usage': j}) for i, j in zip(statuses, tls)]
    def submit_solution(self, task, lang, text):
        if isinstance(text, str): text = text.encode('utf-8')
        sid = self.urls['sid']
        url = self.urls['submit']
        data = []
        data.append(b'"SID"\r\n\r\n'+sid.encode('ascii'))
        data.append(b'"prob_id"\r\n\r\n'+str(task).encode('ascii'))
        data.append(b'"lang_id"\r\n\r\n'+str(lang).encode('ascii'))
        data.append(b'"file"; filename="brute.txt"\r\nContent-Type'
                    b': text/plain\r\n\r\n'+text)
        data.append(b'"action_40"\r\n\r\nSend!')
        import random
        while True:
            x = b'----------'+str(random.randrange(1, 1000000000)).encode('ascii')
            for i in data:
                if x in i: break
            else: break
        data = b'\r\n'.join(b'--'+x+b'\r\nContent-Disposition: form-data; name='+i for i in data)+b'\r\n--'+x+b'--\r\n'
        ans = post(url, data, {'Content-Type': 'multipart/form-data; boundary='+x.decode('ascii'), 'Cookie': self.cookie})
        with self.cache_lock: self.stop_caching()
    def status(self):
        code, headers, data = self._cache_get(self.urls['summary'])
        if b'<input type="submit" name="action_35" value="Change!" />' in data:
            raise EJError("Password change is required.")
        if code != 200:
            raise EJError("Failed to fetch task list")
        ths = [i.split('</th>', 1)[0] for i in data.decode('utf-8').split('<th class="b1">')[1:]]
        w = len(ths)
        if w == 0: return {}
        splitted = data.decode('utf-8').split('<td class="b1">')[1:]
        data = [x.split("</td>")[0] for x in splitted]
        idx = ths.index('Status')
        return collections.OrderedDict((a, b if b != '&nbsp;' else None) for a, b in zip(data[ths.index('Short name')::w], data[idx::w]))
    def scores(self, *, total=None):
        code, headers, data = self._cache_get(self.urls['summary'])
        if b'<input type="submit" name="action_35" value="Change!" />' in data:
            raise EJError("Password change is required.")
        if code != 200:
            raise EJError("Failed to fetch task list")
        data0 = data.decode('utf-8')
        ths = [i.split('</th>', 1)[0] for i in data0.split('<th class="b1">')[1:]]
        w = len(ths)
        splitted = data.decode('utf-8').split('<td class="b1">')[1:]
        data = [x.split("</td>")[0] for x in splitted]
        if 'Score' not in ths: ans = {}
        else: ans = collections.OrderedDict(zip(data[ths.index('Short name')::w], [None if x == '&nbsp;' else int(x) for x in data[ths.index('Score')::w]]))
        if total != None and '<p><big>Total score: ' in data0:
            try: ans[total] = int(data0.split('<p><big>Total score: ', 1)[1].split('</big></p>', 1)[0])
            except (ValueError, IndexError): pass
        return ans
    def compile_error(self, id):
        code, headers, data = self._cache_get(self.urls['protocol'].format(run_id=id))
        if b'<input type="submit" name="action_35" value="Change!" />' in data:
            raise EJError("Password change is required.")
        if code != 200:
            raise EJError("Failed to fetch testing protocol.")
        splitted = data.decode('utf-8').split('<pre>')[1:]
        ans = []
        for i in splitted:
            i = i.split('</pre>')[0]
            i = i.split('<')
            i = i[0] + ''.join(j.split('>', 1)[1] for j in i[1:])
            import html
            ans.append(html.unescape(i))
        return '\n'.join(ans)
    def submission_source(self, id):
        code, headers, data = self._cache_get(self.urls['source'].format(run_id=id))
        rhd = dict(headers)
        if 'html' in rhd['Content-Type'] and b'<input type="submit" name="action_35" value="Change!" />' in data:
            raise EJError("Password change is required.")
        if code != 200 or 'html' in rhd['Content-Type']:
            return None
        return data
    def do_action(self, name, *args):
        code, headers, data = get(self.urls[name], {'Cookie': self.cookie})
        if b'<input type="submit" name="action_35" value="Change!" />' in data:
            raise EJError("Password change is required.")
        return code == 302
    def compiler_list(self, prob_id):
        code, headers, data = self._cache_get(self.urls['submission'].format(prob_id=prob_id))
        if b'<input type="submit" name="action_35" value="Change!" />' in data:
            raise EJError("Password change is required.")
        data = data.decode('utf-8')
        if '<input type="hidden" name="lang_id" value="' in data:
            data = data.split('<input type="hidden" name="lang_id" value="', 1)[1]
            num_id = int(data.split('"', 1)[0])
            lit_id = html.unescape(data.split('</td><td class="b0">', 1)[1].split('</td>', 1)[0])
            short, long = lit_id.strip().split(' - ')
            return [bjtypes.compiler_t(num_id, short, long)]
        try: data = data.split('<select name="lang_id">', 1)[1].split('</select>', 1)[0]
        except IndexError: raise EJError("Failed to fetch language list")
        data = data.split('<option ')[1:]
        ans = [] 
        for i in data:
            a, b = (' '+i).split(' value="', 1)[1].split('"', 1)
            b = b.split('>', 1)[1].split('</option>', 1)[0]
            if not a.isnumeric(): continue
            b, c = html.unescape(b).split(' - ')
            ans.append(bjtypes.compiler_t(int(a), b.strip(), c.strip()))
        return ans
    def submission_stats(self, id):
        code, headers, data = self._cache_get(self.urls['protocol'].format(run_id=id))
        if b'<input type="submit" name="action_35" value="Change!" />' in data:
            raise EJError("Password change is required.")
        data = data.decode('utf-8')
        if '<big>' in data:
            data = '\n\n'.join(i.split('</big>', 1)[0] for i in data.split('<big>')[1:]).split('<')
            data = data[0]+''.join(i.split('>', 1)[1] for i in data[1:])
            ans = {}
            for l in data.split('\n'):
                l = l.split(' ')
                if l[1:4] == ['total', 'tests', 'runs,'] and l[5] == 'passed,' and l[7:] == ['failed.']:
                    ans['tests'] = {}
                    ans['tests']['total'] = int(l[0])
                    ans['tests']['success'] = int(l[4]) 
                    ans['tests']['fail'] = int(l[6])
                elif l[:2] == ['Score', 'gained:']:
                    ans['score'] = int(l[2])
            return (ans, data)
        else:
            return ({}, None)
    def contest_info(self):
        code, headers, data = self._cache_get(self.urls['contest_info'])
        if b'<input type="submit" name="action_35" value="Change!" />' in data:
            raise EJError("Password change is required.")
        data = data.decode('utf-8')
        try: pbs = '\n'.join(html.unescape(i.split('</b></p>', 1)[0]) for i in data.split('<p><b>')[1:])
        except IndexError: pbs = ''
        datas = {}
        for i in data.split('<tr><td class="b0">')[1:]:
            i = i.split('</td></tr>', 1)[0] 
            try: key, value = i.split('<td class="b0">')
            except IndexError: pass
            else: datas[html.unescape(key.split('</td>', 1)[0])] = html.unescape(value)
        data1 = {}
        for k1, k2 in (('server_time', 'Server time:'), ('contest_start', 'Contest start time'), ('contest_duration', 'Duration:')):
            if k2 not in datas: continue
            if datas[k2] == 'Unlimited':
                data1[k1] = math.inf
                continue
            if ' ' in datas[k2]:
                date, s_time = datas[k2].split(' ')
                year, month, day = map(int, date.split('/'))
                hour, minute, second = map(int, s_time.split(':'))
                data1[k1] = time.mktime((year, month, day, hour, minute, second, -1, -1, -1))
            else:
                data1[k1] = 0
                for i in map(int, datas[k2].split(':')):
                    data1[k1] = data1[k1] * 60 + i
        if 'contest_start' in data1 and 'contest_duration' in data1:
            data1['contest_end'] = data1['contest_start'] + data1['contest_duration']
        if 'contest_start' in data1 and 'server_time' in data1:
            data1['contest_time'] = data1['server_time'] - data1['contest_start']
        return (pbs, datas, data1)
    def problem_info(self, id):
        code, headers, data = self._cache_get(self.urls['submission'].format(prob_id=id))
        if b'<input type="submit" name="action_35" value="Change!" />' in data:
            raise EJError("Password change is required.")
        data = data.decode('utf-8')
        if '<table class="line-table-wb">' not in data: return ({}, None)
        data = data.split('<table class="line-table-wb">', 1)[1]
        stats = {}
        data, data2 = data.split('</table>', 1)
        while '<tr><td><b>' in data:
            k, data = data.split('<tr><td><b>', 1)[1].split('</b></td><td>', 1)
            v, data = data.split('</td></tr>', 1)
            v = v.split('<')
            v = v[0]+''.join(i.split('>', 1)[1] for i in v[1:])
            stats[k.rsplit(':', 1)[0].strip()] = html.unescape(v.strip())
        data = data2.split('<form method="post" enctype="multipart/form-data" action="', 1)[0].rsplit('<h3>', 1)[0]
        if data.endswith('</html>\n'):
            data = ''
        return (stats, html2md.html2md(data, self.urls['download_file'].format(prob_id=id, filename=''), self.urls['submission'].format(prob_id=id)))
    def download_file(self, prob_id, filename):
        code, headers, data = self._cache_get(self.urls['download_file'].format(prob_id=prob_id, filename=filename))
        if code == 404:
            raise EJError("File not found.")
        elif code != 200:
            raise EJError("Error downloading.")
        return data
    def clar_list(self):
        code, headers, data = self._cache_get(self.urls['clars'])
        if b'<input type="submit" name="action_35" value="Change!" />' in data:
            raise EJError("Password change is required.")
        ths = [i.split('</th>', 1)[0] for i in data.decode('utf-8').split('<th class="b1">')[1:]]
        w = len(ths)
        splitted = data.decode('utf-8').split('<td class="b1">')[1:]
        data = [x.split("</td>")[0] for x in splitted]
        return [bjtypes.clar_t(i, j) for i, j in zip(map(int, data[ths.index('Clar ID')::w]), map(html.unescape, data[ths.index('Subject')::w]))]
    def submit_clar(self, task, subject, text): 
        if post(self.urls['submit'], {'SID': self.urls['sid'], 'prob_id': task, 'subject': subject, 'text': text, 'action_41': 'Send!'}, {'Cookie': self.cookie})[0] != 302:
            raise EJError("Failed to submit clar")
        with self.cache_lock: self.stop_caching()
    def read_clar(self, id):
        code, headers, data = self._cache_get(self.urls['read_clar'].format(clar_id=id))
        if b'<input type="submit" name="action_35" value="Change!" />' in data:
            raise EJError("Password change is required.")
        data = html.unescape(data.decode('utf-8').split('<pre class="message">', 1)[1].split('</pre>', 1)[0])
        return data.split('\n', 2)[2]
    def _get_samples(self, err):
        if err == None: err = ''
        err = err.strip()
        if "====== Test #" not in err:
            raise EJError("No test cases available")
        err = err[err.find("====== Test #"):]
        lines = iter(err.split('\n'))
        tests = {}
        curr = None
        for line in lines:
            if line.startswith("====== Test #"):
                num = int(line[13:-7])
                curr = tests[num] = {}
            elif line.startswith('--- '):
                line = line[4:-4]
                if ': size ' not in line: continue
                what, size = line.split(': size ')
                size = int(size) + 1
                data = ''
                while len(data) < size:
                    try: data += '\n' + next(lines)
                    except StopIteration: break
                data = data[1:]
                curr[what] = data
        return tests
    def get_samples(self, subm_id):
        return self._get_samples(self.compile_error(subm_id))
    def scoreboard(self):
        code, headers, data = self._cache_get(self.urls['standings'])
        if b'<input type="submit" name="action_35" value="Change!" />' in data:
            raise EJError("Password change is required.")
        if code != 200:
            raise EJError("Failed to fetch scoreboard.")
        teams = data.decode('utf-8').split('<td  class="st_team">')[1:]
        probs = data.decode('utf-8').split('<td  class="st_prob')[1:]
        naux = 0
        if not teams and b'<table border=1 cellspacing=1 celpadding=3>\n  <tr >\n    <th  align=right>Place</th>' in data:
            probs = sum((i.split('<td class="st_prob') for i in data.decode('utf-8').split('<td  align=center')), [])[1:]
            teams = data.decode('utf-8').split('<td  align=left>')[1:]
            naux = 1
        probs = [x.split("</td>", 1)[0] for x in probs]
        teams = [html.unescape(x.split("</td>", 1)[0]) for x in teams]
        try: ntasks = len(probs) // len(teams) - naux
        except ZeroDivisionError: return []
        del teams[-3:]
        del probs[-3*ntasks:]
        probs = iter(probs)
        ans = []
        for i in teams:
            ans.append(({'name': i}, []))
            for j in range(ntasks):
                j = next(probs).split('>', 1)[1]
                if j == '&nbsp;': ans[-1][1].append(None)
                elif j[:1] in ('+', '-'):
                    attempts = int(j[0]+'0'+j[1:])
                    ans[-1][1].append({'attempts': attempts})
                elif j.startswith('<b>') and j.endswith('</b>') and j[3:-4].isnumeric():
                    score = int(j[3:-4])
                    attempts = float('inf')
                    ans[-1][1].append({'score': score, 'attempts': attempts})
                elif j.isnumeric():
                    score = int(j)
                    attempts = float('-inf')
                    ans[-1][1].append({'score': score, 'attempts': attempts})
                else:
                    assert False, repr(j)
            for j in range(naux): next(probs)
        return ans
    def stop_caching(self):
        self._get_cache.clear()
    def contest_list(self):
        if isinstance(self, str): url = self
        else: url = self.urls['contest_list']
        code, headers, data = get(url)
        if code != 200:
            return []
        ans = []
        for i in data.decode('utf-8').split('<td><a href="')[1:]:
            url = html.unescape(i.split('"', 1)[0])
            name = html.unescape(i.split('>', 1)[1].split('<', 1)[0])
            ans.append((name, url, {}))
        return ans
    def change_password(self, oldpwd, newpwd):
        if post(self.urls['submit'], {'SID': self.urls['sid'], 'oldpasswd': oldpwd, 'newpasswd1': newpwd, 'newpasswd2': newpwd, 'action_35': 'Change!'}, {'Cookie': self.cookie})[0] != 302:
            raise EJError("Failed to change password.")



import cmd, sys, traceback, os, shlex
from .http import login_type, login, tasks, submissions, submission_protocol, submit_solution, status, scores, compile_error, submission_source, compiler_list, submission_stats, has_feature, contest_list
from .error import EJError
import ejcli.commands

def isspace(s):
    return not s or s.isspace()

class BruteCMD(cmd.Cmd):
    prompt = 'ejcli> '
    def __init__(self):
        cmd.Cmd.__init__(self)
        self._url = None
        self._cookie = None
    @property
    def url(self):
        if self._url != None: return self._url
        raise EJError("Not logged in")
    @url.setter
    def url(self, val):
        self._url = val
    @property
    def cookie(self):
        if self._cookie != None: return self._cookie
        raise EJError("Not logged in")
    @cookie.setter
    def cookie(self, val):
        self._cookie = val
    def do_login(self, cmd):
        """
        usage: login <login_page_url> <username>

        Login to CMS.
        """
        data = cmd.split(' ')
        if len(data) != 2:
            return self.do_help('login')
        import getpass
        x = getpass.getpass()
        self.url, self.cookie = login(data[0], data[1], x)
    def do_anonlogin(self, cmd):
        """
        usage: anonlogin <login_page_url>

        Perform an anonymous login to CMS.
        """
        data = cmd.split(' ')
        if len(data) != 1:
            return self.do_help('login')
        url = data[0]
        if 'pass' not in login_type(url):
            raise EJError("Anonymous login is not supported")
        self.url, self.cookie = login(url, None, None)
    def do_logout(self, cmd):
        """
        usage: logout

        Logout from CMS.
        """
        if not isspace(cmd):
            return self.do_help('logout')
        self.url = self.cookie = None
    def do_lsc(self, cmd):
        """
        usage: lsc [url]

        List all contests for the given URL, and their corresponding login URLs.
        """
        if isspace(cmd):
            url, cookie = self.url, self.cookie
        else:
            url = cmd
            cookie = None
        data = contest_list(url, cookie)
        print("URL\tTitle")
        for url, title, _ in data:
            print(url+'\t'+title)
    def do_tasks(self, cmd):
        """
        usage: tasks [--verbose]

        Show task list. If --verbose is specified, also show additional information.
        """
        if not isspace(cmd) and cmd.strip() != '--verbose':
            return self.do_help('tasks')
        data = tasks(self.url, self.cookie)
        if cmd.strip() == '--verbose':
            print('Short name\tLong name')
            for i, j, k in data: print(j, k, sep='\t\t')
        else:
            print(*[i[1] for i in data])
    def do_submissions(self, cmd):
        """
        usage: submissions [subm_id]

        Show a list of submissions or detailed log for a single submission.
        """
        if not (isspace(cmd) or cmd.strip().isnumeric()):
            return self.do_help('submissions')
        if isspace(cmd):
            print('Submission ID\tTask\tStatus\t\t\tScore\tTests passed')
            for x in reversed(submissions(self.url, self.cookie)):
                print('%d\t\t%s\t%s\t%s\t%s'%tuple(i if i != None else '' for i in x))
        else:
            print('Test\tVerdict\t\tTime usage\tMemory usage\t...')
            for t, (i, j) in enumerate(submission_protocol(self.url, self.cookie, int(cmd))):
                tl = ''
                if 'time_usage' in j:
                    tl = '%0.3f'%j['time_usage']
                    del j['time_usage']
                ml = ''
                if 'memory_usage' in j:
                    ml = j['memory_usage']
                    del j['memory_usage']
                    if ml < 2**10: ml = '%d B' % ml
                    elif ml < 2**20: ml = '%.1f KiB' % (ml / 2**10)
                    elif ml < 2**30: ml = '%.1f MiB' % (ml / 2**20)
                    elif ml < 2**40: ml = '%.1f GiB' % (ml / 2**30)
                    elif ml < 2**50: ml = '%.1f TiB' % (ml / 2**40)
                    elif ml < 2**60: ml = '%.1f PiB' % (ml / 2**50)
                    else: ml = '%.1f EiB' % (ml / 2**60)
                print('%03d\t%s\t%s\t\t%s\t%s'%(t+1, i, tl, ml, repr(j) if j else ''))
    def do_submit(self, cmd):
        """
        usage: submit <task> <lang_id> <file>

        Submit a new solution.
        """
        sp = cmd.split()
        if len(sp) != 3:
            return self.do_help('submit')
        task_list = tasks(self.url, self.cookie)
        try: task_id = next(i.id for i in task_list if i.short_name == sp[0])
        except StopIteration:
            raise EJError("No such task.")
        if not sp[1].isnumeric():
            raise EJError("lang_id must be a number")
        try:
            with open(sp[2], 'rb') as file:
                data = file.read()
        except FileNotFoundError:
            raise EJError("File not found.")
#       except UnicodeDecodeError:
#           raise EJError("File is binary.")
        submit_solution(self.url, self.cookie, task_id, int(sp[1]), data)
    def do_shell(self, cmd):
        """
        usage: shell <command>

        Run a shell command.
        """
        os.system('bash -c '+shlex.quote(cmd))
    def do_status(self, cmd):
        """
        usage: status [submission_id]

        Show summary.
        """
        if not (isspace(cmd) or cmd.strip().isnumeric()):
            return self.do_help('status')
        if not isspace(cmd):
            splitted = cmd.split()
            if len(splitted) != 1:
                return self.do_help('status')
            for i in submissions(self.url, self.cookie):
                if i.id == int(splitted[0]):
                    print(i.status)
            return
        ans = status(self.url, self.cookie)
        print('Task\tStatus')
        for k, v in list(ans.items()):
            print(k+'\t'+str(v))
    def do_scores(self, cmd):
        """
        usage: scores [subm_id]

        Show score for a solution if subm_id is specified.
        Show scores for each task if subm_id is not specified.
        """
        if isspace(cmd):
            if has_feature(self.url, self.cookie, 'scores', 'total'):
                ans = scores(self.url, self.cookie, total=True)
            else:
                ans = scores(self.url, self.cookie)
            print('Task\tScore')
            for k, v in ans.items():
                if k is not True:
                    print(k+'\t'+str(v))
            if True in ans:
                print('Total score:', ans[True])
        else:
            if len(cmd.split()) != 1:
                return self.do_help('scores')
            try: subm_id = int(cmd)
            except ValueError:
                return self.do_help('scores')
            for i in submissions(self.url, self.cookie):
                if i.id == subm_id:
                    print(i.score)
    def do_source(self, cmd):
        """
        usage: source <submission_id>

        Show program source code for a solution.
        """
        if not cmd.strip().isnumeric():
            return self.do_help('source')
        src = submission_source(self.url, self.cookie, int(cmd))
        if src == None:
            raise EJError('Source code is not available')
        print(src.decode('utf-8', 'replace'))
    def onecmd(self, c):
        try:
            try: return cmd.Cmd.onecmd(self, c)
            except (EJError, SystemExit): raise
            except BaseException:
                raise EJError(traceback.format_exc())
        except EJError as e:
            self.print_error(e)
    def print_error(self, e):
        print(e, file=sys.stderr)
    def __getattr__(self, cmd):
        if not cmd.startswith('do_'): raise AttributeError(cmd)
        cmd = cmd[3:]
        try: __import__('ejcli.commands.' + cmd)
        except ImportError: raise AttributeError(cmd)
        return type(self.__getattr__)(getattr(getattr(ejcli.commands, cmd), 'do_'+cmd), self)
    def get_names(self):
        dir0 = set(dir(type(self)))
        import pkgutil
        try: cmds = list(pkgutil.iter_modules(ejcli.commands.__path__))
        except: pass
        else: dir0 |= {'do_'+i[1] for i in cmds if not i[1].startswith('_')}
        return list(dir0)
    def default(self, cmd):
#       cmd, arg = (cmd.strip()+' ').split(' ', 1)
#       arg = arg.strip()
#       try: __import__('ejcli.commands.' + cmd)
#       except ImportError:
        raise EJError('No such command: ' + cmd.split()[0])
#       return getattr(getattr(ejcli.commands, cmd), 'do_'+cmd)(self, arg)
    def do_EOF(self, cmd=None):
        print(file=sys.stderr)
        sys.exit(0)
    def do_python(self, cmd):
        if cmd and not cmd.isspace():
            exec(compile(cmd, '<string>', 'single'))
        else:
            import code
            code.interact(local={'self': self})
    def do_geterror(self, cmd):
        """
        usage: geterror [--binary] [--kind <kind>] <subm_id>

        Get errors for submission (if any).
        Possible kinds (default order):
           --. Judge comment
            2. Valuer comment
            1. Compilation error
            3. Binary executable file (JJS only)
        """
        sp = cmd.split()
        kwargs = {}
        if sp and sp[0] == '--binary':
            if not has_feature(self.url, self.cookie, 'compile_error', 'binary'):
                raise EJError('This system does not support --binary in geterror.')
            del sp[0]
            kwargs['binary'] = True
        if len(sp) >= 2 and sp[0] == '--kind':
            if not has_feature(self.url, self.cookie, 'compile_error', 'kind'):
                raise EJError('This system does not support --kind in geterror.')
            try: kwargs['kind'] = int(sp[1])
            except ValueError:
                raise EJError('Kind must be a number')
            del sp[:2]
        if len(sp) != 1:
            return self.do_help('geterror')
        try: sp[0] = int(sp[0])
        except ValueError:
            raise EJError('Submission ID must be a number')
        ans = compile_error(self.url, self.cookie, sp[0], **kwargs)
#       except IndexError:
#           print('Success')
        if isinstance(ans, bytes):
            sys.stdout.buffer.raw.write(ans)
        else:
            print(ans)
    def do_compilers(self, cmd):
        """
        usage: compilers <task>

        Get list of all all available compilers
        """
        task = cmd.strip()
        if not task:
            return self.do_help('compilers')
        try:
            prob_id = next(i.id for i in tasks(self.url, self.cookie) if i.short_name == task)
        except StopIteration:
            raise EJError("No such task")
        ans = compiler_list(self.url, self.cookie, prob_id)
        print("ID\tCompiler")
        for i, j, k in ans:
            print(i, j+' - '+k, sep='\t')
    def do_stats(self, cmd):
        """
        usage: stats <subm_id>

        Show statistics for a given submission
        """
        sp = cmd.split()
        if len(sp) != 1:
            return self.do_help('stats')
        if not sp[0].isnumeric():
            raise EJError('Submission ID must be a number')
        stats, stats_str = submission_stats(self.url, self.cookie, int(sp[0]))
        if stats_str != None: print(stats_str.strip())
        import pprint
        pprint.pprint(stats)



from ejcli.http import do_action
from ejcli.error import EJError

def do_control(self, cmd):
    """
    usage: do_control <action> [args]

    `action` can be one of:
        start-virtual
        Start virtual contest.

        stop-virtual
        Stop virtual contest.
    """
    cmd = cmd.strip()
    if cmd == 'start-virtual':
        if not do_action(self.url, self.cookie):
            raise EJError("Failed to start virtual contest")
    elif cmd == 'stop-virtual':
        if not do_action(self.url, self.cookie):
            raise EJError("Failed to stop virtual contest")
    else:
        return self.do_help('control')



import collections
from ejcli.http import tasks
from ejcli.error import EJError
import ejcli._http.types

class TaskAlias:
    def __init__(self, parent):
        self.parent = parent
        self.taskalias_k2v = {}
        self.taskalias_v2k = {}
    def tasks(self):
        return [ejcli._http.types.task_t(i, self.taskalias_k2v.get(j, j), k) for i, j, k in self.parent.tasks()]
    def submissions(self):
        return [ejcli._http.types.submission_t(i, self.taskalias_k2v.get(j, j), *k) for i, j, *k in self.parent.submissions()]
    def status(self):
        return collections.OrderedDict((self.taskalias_k2v.get(k, k), v) for k, v in self.parent.status().items())
    def scores(self):
        return collections.OrderedDict((self.taskalias_k2v.get(k, k), v) for k, v in self.parent.scores().items())
    def __getattr__(self, attr):
        ans = getattr(self.parent, attr)
        return ans
    @classmethod
    def _from_pickled(self, parent, taskalias_k2v, taskalias_v2k):
        ans = self(parent)
        ans.taskalias_k2v = taskalias_k2v
        ans.taskalias_v2k = taskalias_v2k
        return ans
    def __reduce__(self):
        return (self._from_pickled, (self.parent, self.taskalias_k2v, self.taskalias_v2k))

def do_taskalias(self, cmd):
    """
    usage: taskalias <old=new> [old2=new2]...
           taskalias <new>...

    Rename tasks (for your convenience). This does not have any effect for other contestants.
    If using the second prototype, the number of arguments must match the number of tasks.
    """
    cmd = cmd.split()
    eq = ['=' in i for i in cmd]
    if not (cmd and (all(eq) or not any(eq))):
        return self.do_help('taskalias')
    if not isinstance(self.url, TaskAlias): self.url = TaskAlias(self.url)
    if any(eq):
        pairs = [i.split('=', 1) for i in cmd]
    else:
        ts = [i.short_name for i in tasks(self.url, self.cookie)]
        if len(ts) != len(cmd):
            raise EJError("The number of arguments must match the number of tasks.")
        pairs = list(zip(ts, cmd))
    for k, v in pairs:
        try: k1 = self.url.taskalias_v2k[k]
        except KeyError: pass
        else:
            del self.url.taskalias_v2k[k]
            k = k1
        self.url.taskalias_v2k[v] = k
        self.url.taskalias_k2v[k] = v



import urllib.request

class OpenerWrapper:
    def __init__(self, o):
        object.__setattr__(self, '_real_opener', o)
    @classmethod
    def _from_pickled(self, cookies, addheaders):
        import http.cookiejar
        x = http.cookiejar.CookieJar()
        for i in cookies: x.set_cookie(i)
        op = urllib.request.build_opener(urllib.request.HTTPCookieProcessor(x))
        op.addheaders = addheaders
        return self(op)
    def __reduce__(self):
        return (self._from_pickled, (list([i for i in self._real_opener.handlers if isinstance(i, urllib.request.HTTPCookieProcessor)][0].cookiejar), self.addheaders))
    def __getattr__(self, attr):
        return getattr(self._real_opener, attr)
    def __setattr__(self, attr, val):
        setattr(self._real_opener, attr, val)
    def __delattr__(self, attr):
        delattr(self._real_opener, attr)



from ejcli.http import submit_solution, tasks, submissions, compiler_list, may_cache, compile_error
from ejcli.error import EJError
import os.path, shlex, sys

def get_lang_id(self, lang_name, task_id):
    lang_id = [i for i, j, k in compiler_list(self.url, self.cookie, task_id) if j == lang_name]
    try: lang_id, = lang_id
    except ValueError: raise EJError("Unknown language: "+lang_name)
    return lang_id

def get_possible_lang_id(self, lang_names, task_id):
    lang_id = [i for i, j, k in compiler_list(self.url, self.cookie, task_id) if j in lang_names]
    lang_id.append(None)
    return lang_id[0]

def check_exists(fname, options=set()):
    return os.path.exists(fname) or '-exist' in options

def do_asubmit(self, cmd, *, afmt=False):
    """
    usage: asubmit [-w] [-e] [-x <extension>] <task> <lang_id> <file>

    Submit a new solution, using style-fixed version.
    Uses a specific style fixer if -x is specified.
    Waits until testing ends if -w is specified.
    """
    modname = ''
    wait = False
    show_error = False
    sp = shlex.split(cmd)
    if len(sp) not in (range(3, 7) if not afmt else (1, 3)):
        return self.do_help('aformat' if afmt else 'asubmit')
    if not afmt and sp[0] == '-w':
        wait = True
        del sp[0]
    if not afmt and sp[0] == '-e':
        wait = True
        show_error = True
        del sp[0]
    if len(sp) not in ((3, 5) if not afmt else (1, 3)):
        return self.do_help('aformat' if afmt else 'asubmit')
    if sp[0] == '-x':
        modname = sp[1]
        del sp[:2]
        if modname[:1] == '.': modname = 'ejcli.commands.asubmit.format_'+modname[1:]
    if len(sp) != (1 if afmt else 3):
        return self.do_help('aformat' if afmt else 'asubmit')
    try:
        name = sp[-1]
        module = None
        ext = os.path.splitext(name)[1][1:]
        modname, *modargs = modname.split(',')
        modargs = set(modargs)
        if not modname: modname = 'ejcli.commands.asubmit.format_'+ext
        try:
            module = __import__(modname, fromlist=True)
        except ImportError: pass
        if hasattr(module, 'cheats'):
            import ejcli.cheats
            ejcli.cheats.cheating(self)
        if not getattr(module, 'check_exists', check_exists)(name, modargs):
            raise EJError("File not found.")
        if hasattr(module, 'read_file'):
            data = module.read_file(name, modargs)
        else:
            with open(name, 'rb') as file:
                data = file.read()
        if hasattr(module, 'format'):
            data = module.format(data, modargs)
    except UnicodeDecodeError:
        raise EJError("File is binary.")
    if afmt:
        if isinstance(data, str): data = data.encode('utf-8')
        sys.stdout.buffer.write(data)
        sys.stdout.buffer.flush()
        return
    with may_cache(self.url, self.cookie):
        try: task_id = next(i.id for i in tasks(self.url, self.cookie) if i.short_name == sp[0])
        except StopIteration:
            raise EJError("No such task.")
        if not sp[1].isnumeric():
            sp[1] = get_lang_id(self, sp[1], task_id)
        before = submissions(self.url, self.cookie)
        submit_solution(self.url, self.cookie, task_id, int(sp[1]), data)
    after = submissions(self.url, self.cookie)
    if before == after:
        raise EJError("Error while sending.")
    else:
        print('Submission ID is', after[0].id)
        if wait:
            self.do_astatus(str(after[0].id))
            if show_error:
                print(compile_error(self.url, self.cookie, after[0].id))



import setuptools

setuptools.setup(
    name = 'ejcli',
    version = '1.9.3',
    description = 'A testing system console client',
    long_description = open('README.md').read(),
    long_description_content_type = 'text/markdown',
    packages = setuptools.find_packages(),
    entry_points = {
        'console_scripts': ['ejcli = ejcli.__main__:_']
    }
)



import sys

if '--bash' in sys.argv or '--zsh' in sys.argv:
    import ejcli.bashhelper
    ejcli.bashhelper.main()
    exit(0)

from ejcli.cmd import BruteCMD
import cmd, builtins

def input(prompt = ''):
    while True:
        try: return builtins.input(prompt)
        except KeyboardInterrupt: pass

cmd.input = input

print('Type `help` for help.')
print('TIP: use `ejcli --bash` or `ejcli --zsh` to integrate ejcli into your shell')
BruteCMD().cmdloop()



from ejcli.http import tasks, scoreboard
from ejcli.error import EJError

def format_single(data):
    score = data.get('score', None)
    attempts = data.get('attempts', None)
    ans = []
    if score != None: ans.append(str(score))
    if attempts != None and attempts * 0 == 0:
        if attempts == 0: ans.append('+')
        elif attempts > 0: ans.append('+'+str(attempts))
        else: ans.append(str(attempts))
    return ('', '%s', '%s (%s)')[len(ans)] % tuple(ans)

def do_scoreboard(self, cmd):
    """
    usage: scoreboard

    Show current standings.
    """
    cmd = cmd.strip()
    if cmd:
        return self.do_help('scoreboard')
    ts = [i.short_name for i in tasks(self.url, self.cookie)]
    scb = scoreboard(self.url, self.cookie)
    table = [['']+ts]
    for u, i in scb:
        table.append([u.get('name', '')]+['' if j == None else format_single(j) for j in i])
    if table:
        clens = [max(len(j[i]) for j in table) for i in range(len(ts)+1)]
        fmt_s = ' '.join('%%%ds'%i for i in clens)
        for i in table: print(fmt_s % tuple(i))



import subprocess
from ejcli.error import EJError

def check_exists(file, options=set()):
    return True

def read_file(path, options=set()):
    p = subprocess.Popen(path, shell=True, stdout=subprocess.PIPE)
    data = p.stdout.read()
    if p.wait():
        raise EJError("Process terminated with exit code %d"%p.wait())
    return data



import sys, os.path

def read_file(name, options=set(), includes=None):
    name = os.path.realpath(name)
    if includes == None:
        includes = set()
    if name in includes: return ''
    includes.add(name)
    with open(name) as file:
        lines = file.read().split('\n')
    if '-include' not in options:
        for i in range(len(lines)):
            l = lines[i]
            if l.startswith('%include "') and l != '%include "io.inc"':
                p2 = os.path.join(os.path.split(name)[0], eval(l[9:]))
                lines[i] = read_file(p2, includes=includes)
    ans = '\n'.join(lines)
    ans2 = ''
    c = 0
    for i in ans:
        if i == '\n':
            c += 1
            if c >= 3: continue
        else:
            c = 0
        ans2 += i
    includes.remove(name)
    return ans2

def format(original, options=set()):
    if '-indent' in options: return original
    lines = [i.strip() for i in original.split('\n')]
    i = 0
    j = 0
    whole_comment = True
    have_label = False
    while j < len(lines):
        if lines[j] == '':
            if not whole_comment:
                for i in range(i, j):
                    is_label = lines[i].split(';', 1)[0].strip().endswith(':')
                    is_directive = lines[i].split()[0] in ('global', 'section')
                    if is_label: have_label = True
                    if have_label and not (is_label or is_directive):
                        lines[i] = '    ' + lines[i]
            while j < len(lines) and lines[j] == '': j += 1
            i = j
            whole_comment = True
            continue
        if not lines[j].startswith(';'): whole_comment = False
        j += 1
    return '\n'.join(lines)



import threading

class CacheContext:
    def __init__(self, be):
        self.be = be
    def __enter__(self):
        with self.be.cache_lock:
            self.be.caching += 1
    def __exit__(self, *args):
        with self.be.cache_lock:
            self.be.caching -= 1
            if not self.be.caching: self.be.stop_caching()

class LockWrapper:
    def __init__(self):
        self.lock = threading.RLock()
    def __enter__(self):
        self.lock.acquire()
    def __exit__(self, *args):
        self.lock.release()
    def __reduce__(self):
        return (LockWrapper, ())

class Backend:
    def __init__(self):
        self.cache_lock = LockWrapper()
        self.caching = 0
    def may_cache(self):
        return CacheContext(self)
    def stop_caching(self): pass
    @staticmethod
    def login_type(url):
        return ['login', 'pass']
    def contest_list(self):
        return []



import ejcli._http.ejudge, ejcli._http.ejudge.ejfuse, ejcli._http.pcms, ejcli._http.jjs, ejcli._http.informatics, ejcli._http.codeforces, ejcli._http.gcj, ejcli._http.cache, ejcli._http.file, urllib.request
from ejcli._http.ejudge import contest_name
from ejcli.error import EJError
from ejcli._http.types import *

backend_path = [ejcli._http.cache.AggressiveCacheBackend, ejcli._http.file.FileBackend, ejcli._http.ejudge.ejfuse.EJFuse, ejcli._http.jjs.JJS, ejcli._http.informatics.Informatics, ejcli._http.codeforces.CodeForces, ejcli._http.gcj.GCJ, ejcli._http.pcms.PCMS, ejcli._http.ejudge.Ejudge]

def login(url, login, password, **kwds):
    for i in backend_path:
        try: f = i.detect(url)
        except Exception: f = False
        if f:
            return (i(url, login, password, **kwds), True)
    raise EJError("Unknown CMS")

def login_type(url):
    for i in backend_path:
        try: f = i.detect(url)
        except Exception: pass
        if f: return i.login_type(url)
    return []

def _create_wrapper(name):
    def func(*args, **kwds):
        return getattr(args[0], name)(*args[2:], **kwds)
    globals()[name] = func

for _w in ['tasks', 'submissions', 'submission_protocol', 'task_ids', 'submit_solution', 'status', 'scores', 'compile_error', 'submission_source', 'do_action', 'compiler_list', 'submission_stats', 'contest_info', 'problem_info', 'download_file', 'clar_list', 'submit_clar', 'read_clar', 'get_samples', 'may_cache', 'scoreboard', 'change_password', 'locales', 'set_locale']:
    _create_wrapper(_w)

del _create_wrapper, _w

def has_feature(url, cookie, methodname, argname):
    if not hasattr(url, methodname): return False
    m = getattr(url, methodname).__func__.__code__
    return argname in m.co_varnames[:m.co_argcount+m.co_kwonlyargcount]

def contest_list(url, cookie):
    if cookie == None: # anonymous, url is string with url
        for i in backend_path:
            try: f = i.detect(url)
            except Exception: f = False
            if f:
                return i.contest_list(url)
        return []
    else: # non-anonymous, url is the backend object
        return url.contest_list()

def task_list(url, cookie):
    return [i[1] for i in tasks(url, cookie)]

def submission_list(url, cookie):
    sl = submissions(url, cookie)
    return [i[0] for i in sl], [i[1] for i in sl]

def submission_results(url, cookie, subm_id):
    sr = submission_protocol(url, cookie, subm_id)
    return [i[0] for i in sr], ['%0.3f'%i[1]['time_usage'] if 'time_usage' in i[1] else '?.???' for i in sr]

def task_ids(url, cookie):
    return [i[0] for i in tasks(url, cookie)]

def submit(url, cookie, task, lang, text):
    try: task = task_ids(url, cookie)[task]
    except IndexError: return
    submit_solution(url, cookie, task, lang, text)

def submission_status(url, cookie, subm_id):
    for i in submissions(url, cookie):
        if i[0] == subm_id: return i[2]
    return None

def submission_score(url, cookie, subm_id):
    for i in submissions(url, cookie):
        if i[0] == subm_id: return i[3]
    return None

def clars(url, cookie):
    c = clar_list(url, cookie)
    return [i[0] for i in c], [i[1] for i in c]



import urllib.request, urllib.parse, html, json

class PCMSRedirectHandler(urllib.request.HTTPRedirectHandler):
    def redirect_request(self, req, fp, code, msg, headers, newurl):
        newurl = newurl.split(';', 1)[0]
        return urllib.request.HTTPRedirectHandler.redirect_request(self, req, fp, code, msg, headers, newurl)

try: from ejcli._http.openerwr import OpenerWrapper
except ImportError:
    def OpenerWrapper(x): return x

class PCMS:
    @staticmethod
    def detect(url):
        try:
            opener = urllib.request.build_opener(urllib.request.HTTPCookieProcessor, PCMSRedirectHandler)
            req = opener.open(url)
            data = req.read().decode('utf-8')
        except:
            raise
            data = ''
        return '<input type="hidden" name="javax.faces.ViewState" id="j_id1:javax.faces.ViewState:0" value="' in data
    def __init__(self, url, auth=None):
        self.opener = OpenerWrapper(urllib.request.build_opener(urllib.request.HTTPCookieProcessor, PCMSRedirectHandler))
        req = self.opener.open(url)
        if req.geturl().endswith('/login.xhtml'):
            login, password = auth
            req2 = self.opener.open(req.geturl(), urllib.parse.urlencode({'login': 'login', 'login:name': login, 'login:password': password, 'login:submit': 'Login', 'login:login': 'Login', 'javax.faces.ViewState': self.get_view_state(req.read().decode('utf-8'))}).encode('ascii'))
#           except Exception as e: req2 = e
#           print(req2.read().decode('utf-8'))
        else: req2 = req
        assert not req2.geturl().endswith('/login.xhtml'), "Login failed"
        self.base_url = req2.geturl().rsplit('/', 1)[0]
    @staticmethod
    def get_view_state(data):
        return data.split('<input type="hidden" name="javax.faces.ViewState" id="', 1)[1].split('"', 3)[2]
    def set_locale(self, loc):
        url = self.base_url
        if url.endswith('/party'): url = url[:-6]
        url += '/set-locale.xhtml'
        self.opener.open(url+'?locale-name='+loc).read()
    def contest_list(self):
        data = self.opener.open(self.base_url + '/contests.xhtml').read().decode('utf-8')
        names = list(map(html.unescape, (i.split('"', 1)[0] for i in data.split('<a href="#" title="')[1:])))
        return names, self.get_view_state(data)
    def select_contest(self, index, viewstate):
        self.opener.open(self.base_url + '/contests.xhtml', urllib.parse.urlencode({'j_idt25': 'j_idt25', 'j_idt25:j_idt26:%d:j_idt28'%index: 'j_idt25:j_idt26:%d:j_idt28'%index, 'javax.faces.ViewState': viewstate}).encode('ascii'))
    def get_info(self):
        data = self.opener.open(self.base_url + '/information.xhtml').read().decode('utf-8')
        props_s = data.split('<table class="properties">', 1)[1].split('</table>', 1)[0]
        keys = [] 
        values = []
        for i in props_s.split('<td class="key">')[1:]:
            keys.append(html.unescape(i.split('</td>', 1)[0]))
        for i in props_s.split('<td class="value">')[1:]:
            values.append(i.split('</td>', 1)[0])
        props = dict(zip(keys, values))
        return props
    def get_messages(self):
        data = self.opener.open(self.base_url + '/information.xhtml').read().decode('utf-8')
        messages = []
        try: messages_s = data.split('<table class="messages list">', 1)[1].split('</table>', 1)[0]
        except IndexError: return []
        for i in messages_s.replace('</td>\n<td class="', '</td><td class="').split('</td><td class="')[1:]:
            i = i.split('</td>', 1)[0]
            t, s = i.split('">', 1)
            messages.append((t, html.unescape(s)))
        return messages
    @staticmethod
    def get_list_content(prob):
        data = []
        flag = False
        for subms in prob.split('<tr class="'):
            kind = subms.split('"', 1)[0]
            for subm in subms.split('<tr>'):
                if not flag:
                    flag = True
                    continue
                subm = subm.split('</tr>', 1)[0]
                subm_d = {}
                data.append((kind, subm_d))
                kind = None
                for i in subm.split('<td ')[1:]:
                    cls, dat = i.split('">', 1)
                    try: cls = cls.split('class="', 1)[1].split('"', 1)[0]
                    except IndexError: cls = None
                    dat = html.unescape(dat.split('</td>', 1)[0])
                    subm_d[cls] = dat
                    try: del subm_d['pad']
                    except KeyError: pass
        return data
    def get_submissions(self, filter=True):
        data = self.opener.open(self.base_url + '/runs.xhtml').read().decode('utf-8')
        try: runs_s = data.split('<table class="runs list">', 1)[1].split('</table>', 1)[0]
        except IndexError: runs_s = ''
        ans = {}
        for prob in runs_s.split('<tbody id="problem-')[1:]:
            prob = prob.split('</tbody>', 1)[0]
            ans[prob.split('"', 1)[0]] = data = [(i[4 if filter else 0:], j) for i, j in self.get_list_content(prob) if i.startswith('run ') or not filter]
            for i in data:
                subm_d = i[1]
                try: subm_d['source'] = urllib.parse.urljoin(self.base_url, subm_d['source'].split('"')[1])
                except KeyError: pass
        return ans
    def get_submit(self):
        data = self.opener.open(self.base_url + '/submit.xhtml').read().decode('utf-8')
        langs_s = data.split('<select id="submit:language" name="submit:language" class="w100" size="1">', 1)[1].split('</select>', 1)[0]
        langs = {}
        for i in langs_s.split('<option value="')[1:]:
            val, name = i.split('</option>', 1)[0].split('">', 1)
            val = val.split('"', 1)[0]
            name = html.unescape(name)
            if name.endswith('\u200e'):
                name = name[:-1]
            langs[val] = name
        probs = []
        for i in data.split('<div id="problem-')[1:]:
            prob, rest = i.split('"', 1)
            rest = rest.split('">', 1)[0]
#           if rest.endswith(' class="allowed'):
            probs.append(prob)
        jsonp = json.loads(data.split('<script type="text/javascript">submits.init(', 1)[1].split(');</script>', 1)[0])
        return (probs, langs, jsonp, self.get_view_state(data))
    def submit(self, task, lang, text, vs, ext='.txt'):
        data = []
        data.append(b'"submit"\r\n\r\nsubmit')
        data.append(b'"submit:problem"\r\n\r\n'+task.encode('utf-8'))
        data.append(b'"submit:language"\r\n\r\n'+str(lang).encode('utf-8'))
        data.append(b'"submit:file"; filename="brute'+ext.encode('utf-8')+b'"\r\nContent-Type'+
                    b': text/plain\r\n\r\n'+text)
        data.append(b'"submit:submitSolution"\r\n\r\nSubmit solution')
        data.append(b'"javax.faces.ViewState"\r\n\r\n'+vs.encode('utf-8'))
        import random
        while True:
            x = b'----------'+str(random.randrange(1, 1000000000)).encode('ascii')
            for i in data:
                if x in i: break
            else: break
        data = b'\r\n'.join(b'--'+x+b'\r\n'+b'Content-Disposition: form-data; name='+i for i in data)+b'\r\n--'+x+b'--\r\n'
#       print(data)
        assert self.opener.open(urllib.request.Request(self.base_url + '/submit.xhtml',
                                data=data,
                                headers={'Content-Type': 'multipart/form-data; boundary='+x.decode('ascii')},
                                method='POST')).geturl() == self.base_url + '/submitDone.xhtml'
    def get_protocol(self, task, attempt):
        req = self.opener.open(self.base_url+'/feedback.xhtml?'+urllib.parse.urlencode({'problem': task, 'attempt': attempt}))
        if req.geturl().endswith('/runs.xhtml'):
            return None
        data = req.read().decode('utf-8')
#       print(data)
        return self.get_list_content(data)
    def get_compile_error(self, task, attempt):
        data = self.opener.open(self.base_url+'/runs.xhtml').read().decode('utf-8')
        try: data = data.split('<tr id="ce-%s_%d" style="display: none"><td class="pad"></td><td colspan="7"><pre>'%(task, attempt), 1)[1].split('</pre>', 1)[0]
        except IndexError: return None
        return html.unescape(data)
    def get_source(self, task, attempt):
        req = self.opener.open(self.base_url+'/sources.xhtml?problem=%s&attempt=%d'%(task, attempt))
        if req.geturl().endswith('/runs.xhtml'):
            return None
        req = req.read()
        try: url = urllib.request.urljoin(self.base_url+'/sources.xhtml', html.unescape((b'downloadFile?token='+req.split(b'<a href="downloadFile?token=', 1)[1].split(b'"', 1)[0]).decode('utf-8', 'replace')))
        except IndexError: pass
        else:
            return self.opener.open(url).read()
        try: return html.unescape(req.split(b'<pre dir="ltr" ', 1)[1].split(b'>', 1)[1].split(b'</pre>', 1)[0].decode('latin-1')).encode('latin-1')
        except IndexError: return None
    def get_links(self):
        req = self.opener.open(self.base_url+'/links.xhtml')
        if not req.geturl().endswith('/links.xhtml'):
            return None
        return req.read().decode('utf-8', 'replace').split('</tr>\n</tbody>\n</table>\n', 1)[1].split('<input type="hidden" ', 1)[0], self.base_url+'/links.html'
    def get_clars(self):
        data = self.opener.open(self.base_url + '/questions.xhtml').read().decode('utf-8')
        probs = []
        for i in data.split('<div id="problem-')[1:]:
            prob, rest = i.split('"', 1)
            rest = rest.split('">', 1)[0]
            probs.append(prob)
        clars = []
        for i in self.get_list_content(data):
            if 'time' in i[1]:
                clars.append(i)
        return (probs, clars, self.get_view_state(data))
#(['A', 'B', 'C', 'D'], [(None, {'time': '275:33', 'text': 'C. Прогулка по Бруклину', 'text pre': 'Север сверху?', 'type': 'Yes', 'answer pre': ''}), (None, {'time': '271:02', 'text': 'A. Дела по дому', 'text pre': 'blablabla\nblablabla', 'type': 'No comments', 'answer pre': ''}), (None, {'time': '270:14', 'text': 'A. Дела по дому', 'text pre': 'blablabla\r\nblablabla', 'type': 'No comments', 'answer pre': ''})], '3763962949909805199:-2195426587356402460'
    def submit_clar(self, task, subject, text, vs):
        assert self.opener.open(self.base_url+'/questions.xhtml', urllib.parse.urlencode({'questionForm': 'questionForm', 'questionForm:problem': task, 'questionForm:question': (subject+'\n'+text).strip(), 'questionForm:askQuestion': 'Ask question', 'javax.faces.ViewState': vs}).encode('ascii')).geturl() == self.base_url+'/questions.xhtml'
    def monitor(self):
        data = self.opener.open(self.base_url + '/monitor.xhtml').read().decode('utf-8')
        probs = [i.split('"', 2)[-1].split('>', 1)[-1].split('<', 1)[0] for i in data.split('<th class="problem" ')[1:]]
        scb = []
        for i in data.split('<tr class="row')[1:]:
            i = i.split('>', 1)[1].split('</tr>', 1)[0]
            info = {}
            problems = []
            for j in i.split('<td class="')[1:]:
                cls, j = j.split('"', 1)
                j = j.split('>', 1)[1].split('</td>', 1)[0]
                if cls.startswith('problem '):
                    flags = cls[8:].split()
                    print(repr(j))
                    score, *penalty = j.split('</div>', 1)[0].split('<div>', 1)
                    print((score, penalty))
                    if not penalty: penalty = None
                    else: penalty, = penalty
                    problems.append((flags, score.strip() or None, penalty.strip() if penalty else None))
                else:
                    info[cls] = j
            scb.append((info, problems))
        return (probs, scb)
